* {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
}

body {
    line-height: 1.7em;
    color: #7f8c8d;
    font-size: 13px;
    font-family: sans-serif;
}

.pure-button {
    background-color: #1f8dd6;
    color: white;
    padding: 0.5em 2em;
    border-radius: 5px;
}

a.pure-button-primary {
    background: white;
    color: #1f8dd6;
    border-radius: 5px;
    font-size: 120%;
}

.splash-container {
    background: #1f8dd6;
    z-index: 1;
    overflow: hidden;
    width: 100%;
    height: 100%;
    top: 0;
    left: 0;
    position: fixed !important;
}

.splash {
    /* absolute center .splash within .splash-container */
    width: 100%;
    height: 50%;
    margin: auto;
    position: absolute;
    top: 100px; left: 0; bottom: 0; right: 0;
    text-align: center;
}

/* This is the subheading that appears on the blue section */
.splash-head {
    font-size: 120%;
    color: white;
    letter-spacing: 0.05em;
    opacity: 0.8;
    text-transform: uppercase;
}

.splash-message {
    color: white;
    letter-spacing: 0.05em;
    margin: 3em;

}

.splash-error {
    color: #000923;
    margin: 3em;
}

@media (min-width: 48em) {

    /* We increase the body font size */
    body {
        font-size: 16px;
    }

    /* We decrease the width of the .splash, since we have more width
    to work with */
    .splash {
        width: 50%;
        height: 50%;
    }

    .splash-head {
        font-size: 250%;
    }

}
